@@ -59,6 +59,8 @@ function EAction(guiAction) {
     this.waitingForContextMenu = false;
 
     this.optOutRelativeZeroResume = false;
+
+    this.resuming = false;
 }
 
 EAction.prototype = new RActionAdapter();
@@ -249,7 +251,7 @@ EAction.prototype.initState = function() {
 };
 
 /**
- * Called whenver the action resumes its operation, for example after it was
+ * Called whenever the action resumes its operation, for example after it was
  * temporary suspended for another action.
  */
 EAction.prototype.resumeEvent = function() {
@@ -283,7 +285,7 @@ EAction.prototype.resumeEvent = function() {
 };
 
 /**
- * Called whenver the action is suspended, for example if another action is
+ * Called whenever the action is suspended, for example if another action is
  * started while this action is active.
  */
 EAction.prototype.suspendEvent = function() {
@@ -356,6 +358,8 @@ EAction.prototype.showUiOptions = function(resume, restoreFromSettings) {
         return;
     }
 
+    this.resuming = true;
+
     this.optionWidgetActions = [];
     for (var i = 0; i < this.uiFile.length; ++i) {
         var uiFile = this.uiFile[i];
@@ -393,7 +397,7 @@ EAction.prototype.showUiOptions = function(resume, restoreFromSettings) {
 
         // automatically add separator to toolbar:
         var a = optionsToolBar.addSeparator();
-        a.objectName = "LastSeparator";
+        a.objectName = "LastSeparator" + i;
         this.optionWidgetActions.push(a);
 
         // give action a chance to initialize toolbar widgets that cannot
@@ -408,6 +412,8 @@ EAction.prototype.showUiOptions = function(resume, restoreFromSettings) {
 
     // hide options tool bar widgets which are shown in a dialog instead:
     this.hideOptionsToolBarWidgets();
+
+    this.resuming = false;
 };
 
 /**
@@ -749,18 +755,24 @@ EAction.prototype.keyReleaseEvent = function(event) {
  */
 EAction.prototype.setCrosshairCursor = function() {
     if (isNull(EAction.crossCursor)) {
-        var bitmap, mask
-        if (RSettings.getDevicePixelRatio()===2 && RS.getSystemId()!=="osx") {
-            bitmap = new QBitmap(EAction.includeBasePath + "/CrosshairCursor@2x.png", "PNG");
-            mask = new QBitmap(EAction.includeBasePath + "/CrosshairCursorMask@2x.png", "PNG");
-            //bitmap.setDevicePixelRatio(2);
-            //mask.setDevicePixelRatio(2);
-            EAction.crossCursor = new QCursor(bitmap, mask, 30, 30);
+
+        if (RSettings.getBoolValue("GraphicsView/SystemCursors", false)===true) {
+            EAction.crossCursor = new QCursor(Qt.CrossCursor);
         }
         else {
-            bitmap = new QBitmap(EAction.includeBasePath + "/CrosshairCursor.png", "PNG");
-            mask = new QBitmap(EAction.includeBasePath + "/CrosshairCursorMask.png", "PNG");
-            EAction.crossCursor = new QCursor(bitmap, mask, 15, 15);
+            var bitmap, mask
+            if (RSettings.getDevicePixelRatio()===2 && RS.getSystemId()!=="osx") {
+                bitmap = new QBitmap(EAction.includeBasePath + "/CrosshairCursor@2x.png", "PNG");
+                mask = new QBitmap(EAction.includeBasePath + "/CrosshairCursorMask@2x.png", "PNG");
+                //bitmap.setDevicePixelRatio(2);
+                //mask.setDevicePixelRatio(2);
+                EAction.crossCursor = new QCursor(bitmap, mask, 30, 30);
+            }
+            else {
+                bitmap = new QBitmap(EAction.includeBasePath + "/CrosshairCursor.png", "PNG");
+                mask = new QBitmap(EAction.includeBasePath + "/CrosshairCursorMask.png", "PNG");
+                EAction.crossCursor = new QCursor(bitmap, mask, 15, 15);
+            }
         }
     }
 
@@ -991,7 +1003,7 @@ EAction.getMenu = function(title, objectName, initFunction) {
  * icon.
  * 
  * \param menu Parent menu.
- * \param sortOrder Sort oder among other submenus. All
+ * \param sortOrder Sort order among other submenus. All
  *      submenus within the same menu are ordered by this number.
  * \param title Translated title of the submenu. E.g. qsTr("&Line").
  * \param objectName Object name to use for the menu. Used mainly for
@@ -1091,37 +1103,6 @@ EAction.getToolBar = function(title, objectName, toolBarArea, category, before)
 
     if (isNull(tb)) {
         tb = new QToolBar(title);
-
-        // style tool buttons in options toolbar:
-        if (RSettings.isQt(5)) {
-            if (!RSettings.hasCustomStyleSheet()) {
-                if (RSettings.hasDarkGuiBackground()) {
-                    tb.setStyleSheet(
-                          "QToolButton {"
-                             // make sure unchecked button renders with same size as checked
-                             // prevents things from moving around when checking / unchecking
-                        + "  border: 1px solid transparent;"
-                        + "} "
-                        + "QToolButton:checked { "
-                        + "  border-top: 1px solid #161616;"
-                        + "  border-left: 1px solid #161616;"
-                        + "  border-bottom: 1px solid #777777;"
-                        + "  border-right: 1px solid #777777;"
-                        + "  background-color: #222222;"
-                        + "}");
-                }
-                else {
-                    tb.setStyleSheet(
-                        "QToolButton {" +
-                        "  border: 1px solid transparent;" +
-                        "} " +
-                        "QToolButton:checked { " +
-                        "  border:1px solid #7f7f7f; " +
-                        "  background: qlineargradient(x1:0 y1:0, x2:0 y2:1 stop:0 #c0c0c0, stop:0.1 #8a8a8a stop:0.2 #a3a3a3 stop:1 transparent); " +
-                        "}");
-                }
-            }
-        }
         tb.objectName = objectName;
         if (!isNull(category)) {
             tb.setProperty("Category", category);
@@ -1379,6 +1360,8 @@ EAction.addGuiActionTo = function(action, iface, addToMenu, addToToolBar,
     if (isNull(addSeparator)) {
         addSeparator = false;
     }
+    var addToUserToolBar1 = false;
+    var addToUserToolBar2 = false;
 
     if (className.length!==0) {
         var key = className + "/VisibleInMenu";
@@ -1396,6 +1379,16 @@ EAction.addGuiActionTo = function(action, iface, addToMenu, addToToolBar,
         if (!RSettings.hasValue(key)) {
             RSettings.setValue(key, addToCadToolBar);
         }
+        key = className + "/VisibleInUserToolBar1";
+        addToUserToolBar1 = RSettings.getBoolValue(key, addToUserToolBar1);
+        if (!RSettings.hasValue(key)) {
+            RSettings.setValue(key, addToUserToolBar1);
+        }
+        key = className + "/VisibleInUserToolBar2";
+        addToUserToolBar2 = RSettings.getBoolValue(key, addToUserToolBar2);
+        if (!RSettings.hasValue(key)) {
+            RSettings.setValue(key, addToUserToolBar2);
+        }
     }
 
     if (isNull(iface)) {
@@ -1455,6 +1448,21 @@ EAction.addGuiActionTo = function(action, iface, addToMenu, addToToolBar,
             }
         }
     }
+
+    var appWin = EAction.getMainWindow();
+    if (addToUserToolBar1) {
+        var tb = appWin.findChild("UserToolBar1");
+        if (!isNull(tb)) {
+            action.addToToolBar(tb);
+        }
+    }
+
+    if (addToUserToolBar2) {
+        var tb = appWin.findChild("UserToolBar2");
+        if (!isNull(tb)) {
+            action.addToToolBar(tb);
+        }
+    }
 };
 
 //EAction.addGuiActionToWidgets = function(action, widgetNames) {
@@ -1638,8 +1646,16 @@ EAction.getScales = function(unit) {
                 '1" = 512"', '1" = 1024"', '1" = 2048"', '1" = 4096"',
                 '1/4096" = 1\'-0"', '1/2048" = 1\'-0"', '1/1024" = 1\'-0"',
                 '1/512" = 1\'-0"', '1/256" = 1\'-0"', '1/128" = 1\'-0"',
-                '1/64" = 1\'-0"', '1/32" = 1\'-0"', '1/16" = 1\'-0"',
-                '1/8" = 1\'-0"', '1/4" = 1\'-0"', '3/4" = 1\'-0"', '1/2" = 1\'-0"',
+                '1/64" = 1\'-0"', 
+                '1/32" = 1\'-0"', 
+                '1/16" = 1\'-0"',
+                '3/32" = 1\'-0"', 
+                '1/8" = 1\'-0"', 
+                '3/16" = 1\'-0"', 
+                '1/4" = 1\'-0"', 
+                '3/8" = 1\'-0"',
+                '1/2" = 1\'-0"',
+                '3/4" = 1\'-0"', 
                 '1" = 1\'-0"', '2" = 1\'-0"', '3" = 1\'-0"', '4" = 1\'-0"',
                 '5" = 1\'-0"', '6" = 1\'-0"', '7" = 1\'-0"', '8" = 1\'-0"',
                 '9" = 1\'-0"', '10" = 1\'-0"', '11" = 1\'-0"');
@@ -1664,7 +1680,7 @@ EAction.prototype.getAuxPreview = function() {
 };
 
 /**
- * Called by updatePreview. Implementations must return an array of enitity IDs
+ * Called by updatePreview. Implementations must return an array of entity IDs
  * which should be highlighted.
  */
 EAction.prototype.getHighlightedEntities = function() {
@@ -1758,6 +1774,13 @@ EAction.prototype.pickCoordinate = function(event, preview) {
  * Default implementation. Calls pickCoordinate(event, false)
  */
 EAction.prototype.coordinateEvent = function(event) {
+    // don't let invalid positions go through for most cases (this is overridden for DefaultAction):
+    var pos = event.getModelPosition();
+    if (!isValidVector(pos)) {
+        EAction.handleUserWarning(qsTr("Invalid position"));
+        return;
+    }
+
     this.pickCoordinate(event, false);
 };
 
@@ -1930,7 +1953,12 @@ EAction.setProgressEnd = function() {
 EAction.assertEditable = function(entity, quiet) {
     if (!entity.isEditable()) {
         if (!quiet) {
-            EAction.handleUserWarning(qsTr("Entity is on a locked layer."));
+            if (!entity.isInWorkingSet()) {
+                EAction.handleUserWarning(qsTr("Entity is not in working set."));
+            }
+            else {
+                EAction.handleUserWarning(qsTr("Entity is on a locked layer."));
+            }
         }
         return false;
     }
@@ -1939,7 +1967,7 @@ EAction.assertEditable = function(entity, quiet) {
 };
 
 /**
- * Allows all actions to handle property change events. This is neccessary to
+ * Allows all actions to handle property change events. This is necessary to
  * ensure that properties can be changed even if a tool is active (e.g. while
  * drawing lines).
  */
@@ -2192,6 +2220,28 @@ EAction.getEntityId = function(di, action, event, preview, selectable) {
     return ret;
 };
 
+EAction.prototype.isEntitySnappable = function(e) {
+    if (!isEntity(e)) {
+        return false;
+    }
+
+    var layerId = e.getLayerId();
+    var data = e.getData();
+    var doc = data.getDocument();
+    if (isNull(doc)) {
+        return false;
+    }
+
+    return doc.isLayerSnappable(layerId);
+};
+
+EAction.prototype.isEntityEditable = function(e) {
+    if (!isEntity(e)) {
+        return false;
+    }
+    return e.isEditable();
+};
+
 /**
  * Some common, shared translated warnings:
  */
@@ -2223,6 +2273,10 @@ EAction.warnNotLineArcCircle = function() {
     EAction.handleUserWarning(qsTr("Entity is not a line, arc or circle."));
 };
 
+EAction.warnNotLineArcSplineEllipseArcPolyline = function() {
+    EAction.handleUserWarning(qsTr("Entity is not a line, arc, spline, ellipse arc or polyline."));
+};
+
 EAction.warnNotArcCircleEllipse = function() {
     EAction.handleUserWarning(qsTr("Entity is not an arc, circle or ellipse."));
 };
